@@ -1002,8 +1002,8 @@ class PopTable(pd.DataFrame):
 
     def get_pop_groups(self):
         '''
-        Get self.__pop_sex and self.__pop_age, the names of sex and age
-        columns.
+        Get the list of self.__pop_sex and self.__pop_age, the names of sex 
+        and age columns.
 
         Usage
         -----
@@ -1048,6 +1048,8 @@ class PopTable(pd.DataFrame):
         Get the vector I := |min(pop_end, 0)|. That is, get the amount of
         which each record requires so that the respective end-of-period
         population becomes nonnegative.
+        Provide a list of `str` to `estime2.options.age.mask` so that the
+        method masks (or ignores) negative values at selected ages.
 
         Usage
         -----
@@ -1058,13 +1060,21 @@ class PopTable(pd.DataFrame):
         If a record has -3 as its end-of-period population, then this
         method will return 3; if a record has a nonnegative end-of-period
         population (e.g. 0, 1, or bigger), then the method returns 0.
+        If `self` has 3 negative end-of-period population records at age 1,
+        2, and 28, and if the user sets ['1', '2'] to `age.mask`, then the
+        I-vector that is returned by this method will return 0's at age 1 
+        and 2, and only non-zero age will be 28.
         '''
 
         result = self.calculate_pop()
+        pop_age = self.get_pop_groups()[1]
         I = result[self.__pop_end].apply(lambda x: abs(min(x, 0)))
         del result[self.__pop_end]
         result['I'] = I
 
+        age_mask = get_option('age.mask')
+        set_0_at_masked_ages(result, pop_age, age_mask, 'I')
+
         return result
 
     def get_J(self, comp: Optional[str] = None):
@@ -1112,6 +1122,34 @@ class PopTable(pd.DataFrame):
 
         return self.__is_subprov
 
+def calculate_ages_to_counter_in_pop(
+    to_counter_age: List[int],
+    comp_in_comp_end: bool
+):
+    '''
+    Return the list of minimum and maximum ages to receive 
+    counter-adjustments in the end-of-period population based on 
+    `to_counter_age`.
+
+    Arguments
+    ---------
+    * `to_counter_age`: a list of two ints; the minimum and maximum 
+        neighbouring ages of the component.
+    * `comp_in_comp_end`: a bool; is that component end-of-period?
+    '''
+
+    to_counter_age_min = to_counter_age[0]
+    to_counter_age_max = to_counter_age[1]
+    age_max = get_option('age.max')
+    result =\
+        [to_counter_age_min, to_counter_age_max] if comp_in_comp_end \
+        else [
+            to_counter_age_min + 1, 
+            min(to_counter_age_max + 1, age_max + 1)
+        ]
+
+    return result
+
 def calculate_ages_to_modify_and_counter(
     problematic_age: Age,
     comp_in_comp_end: bool
@@ -1126,7 +1164,10 @@ def calculate_ages_to_modify_and_counter(
     method. It computes the age(s) of records to be modified. For example,
     if the negative end-of-period population is detected at age 97, and
     the component is NOT one of the end-of-period components, then the 
-    method will select the age 96 
+    method will select the age 96 as an age to be modified in the
+    component. Depending on the value of `method_use.age_floor`, 
+    `age.floor`, `age.down_to`, and `method_use.old_neigh`, the 
+    neighbouring ages will vary.
 
     Arguments
     ---------
@@ -1139,11 +1180,11 @@ def calculate_ages_to_modify_and_counter(
     >>> # age.down_to == 40
     >>> # age.floor == 50
     >>> # age.max == 99
-    >>> age1 = Age(97)
-    >>> age2 = Age('100+') # maximum age
+    >>> age97 = Age(97)
+    >>> age100p = Age('100+') # maximum age
     >>> set_option('method_use.age_floor', False)
     >>> result1 = calculate_ages_to_modify_and_counter(
-    ...     age1, 
+    ...     age97, 
     ...     comp_in_comp_end = False
     ... )
     >>> result1 == {
@@ -1152,7 +1193,7 @@ def calculate_ages_to_modify_and_counter(
     ... }
     True
     >>> result2 = calculate_ages_to_modify_and_counter(
-    ...     age1, 
+    ...     age97, 
     ...     comp_in_comp_end = True
     ... )
     >>> result2 == {
@@ -1161,7 +1202,7 @@ def calculate_ages_to_modify_and_counter(
     ... }
     True
     >>> result3 = calculate_ages_to_modify_and_counter(
-    ...     age2, 
+    ...     age100p, 
     ...     comp_in_comp_end = False
     ... )
     >>> result3 == {
@@ -1170,7 +1211,7 @@ def calculate_ages_to_modify_and_counter(
     ... }
     True
     >>> result4 = calculate_ages_to_modify_and_counter(
-    ...     age2, 
+    ...     age100p, 
     ...     comp_in_comp_end = True
     ... )
     >>> result4 == {
@@ -1181,7 +1222,7 @@ def calculate_ages_to_modify_and_counter(
     >>> set_option('method_use.age_floor', True)
     >>> set_option('age.floor', 50)
     >>> result5 = calculate_ages_to_modify_and_counter(
-    ...     age1, 
+    ...     age97, 
     ...     comp_in_comp_end = False
     ... )
     >>> result5 == {
@@ -1190,7 +1231,7 @@ def calculate_ages_to_modify_and_counter(
     ... }
     True
     >>> result6 = calculate_ages_to_modify_and_counter(
-    ...     age1, 
+    ...     age97, 
     ...     comp_in_comp_end = True
     ... )
     >>> result6 == {
@@ -1207,21 +1248,88 @@ def calculate_ages_to_modify_and_counter(
     ...     'age.to_counter': [34, 73]
     ... }
     True
+    >>> set_option('method_use.old_neigh', True)
+    >>> age7 = Age(7)
+    >>> result8 = calculate_ages_to_modify_and_counter(
+    ...     age7, 
+    ...     comp_in_comp_end = False
+    ... )
+    >>> result8 == {
+    ...     'age.to_modify': 6,
+    ...     'age.to_counter': [7, 50]
+    ... }
+    True
+    >>> result9 = calculate_ages_to_modify_and_counter(
+    ...     age7, 
+    ...     comp_in_comp_end = True
+    ... )
+    >>> result9 == {
+    ...     'age.to_modify': 7,
+    ...     'age.to_counter': [8, 50]
+    ... }
+    True
+    >>> set_option('method_use.age_floor', False)
+    >>> result10 = calculate_ages_to_modify_and_counter(
+    ...     age7, 
+    ...     comp_in_comp_end = False
+    ... )
+    >>> result10 == {
+    ...     'age.to_modify': 6,
+    ...     'age.to_counter': [7, 46]
+    ... }
+    True
+    >>> result11 = calculate_ages_to_modify_and_counter(
+    ...     age7, 
+    ...     comp_in_comp_end = True
+    ... )
+    >>> result11 == {
+    ...     'age.to_modify': 7,
+    ...     'age.to_counter': [8, 47]
+    ... }
+    True
     '''
 
+    # Get options
     age_down_to = get_option('age.down_to')
     age_floor = get_option('age.floor')
+    age_max = get_option('age.max')
     use_age_floor = get_option('method_use.age_floor')
     age_is_max = problematic_age.is_max()
-    to_modify_age_str = str(problematic_age.get_showing_age())
+    showing_age = problematic_age.get_showing_age()
+
+    # Enable/disable old_neigh forcefully depending on problematic_age
+    if age_is_max:
+        use_old_neigh = False
+    elif showing_age == 0:
+        use_old_neigh = True
+    else:
+        use_old_neigh = get_option('method_use.old_neigh')
+
+    # Get the modification age
+    to_modify_age_str = str(showing_age)
     to_modify_age_int = int(to_modify_age_str) - 1
     if comp_in_comp_end:
         to_modify_age_int += 1
-    
-    to_counter_age_max = to_modify_age_int - 1
-    max_of_either = max(-1, to_modify_age_int - age_down_to)
-    to_counter_age_min = min(age_floor, max_of_either) if use_age_floor \
-        else max_of_either
+
+    # Get the min and max neighbouring ages
+    if use_old_neigh:
+        to_counter_age_min = to_modify_age_int + 1
+        min_of_either = min(
+            to_modify_age_int + age_down_to, 
+            age_max + 1
+        )
+        to_counter_age_max =\
+            max(age_floor, min_of_either) if use_age_floor \
+            else min_of_either
+    else:
+        to_counter_age_max = to_modify_age_int - 1
+        max_of_either = max(
+            0 if comp_in_comp_end else -1, 
+            to_modify_age_int - age_down_to
+        )
+        to_counter_age_min =\
+            min(age_floor, max_of_either) if use_age_floor \
+            else max_of_either
 
     to_modify_age_int_lst = None
     result = {}
@@ -1270,16 +1378,6 @@ def calculate_absorbable_in_other_comp(
             [cols_required]
         
         # Absorbables at neighbouring ages (counter-adjusted records)
-        df_peal_ca = None
-        if other_comp_in_comp_end:
-            df_peal_ca = otbl_pop_end\
-                .loc[lambda df: df[pop_age] >= to_counter_age_min]\
-                .loc[lambda df: df[pop_age] <= to_counter_age_max]            
-        else:
-            df_peal_ca = otbl_pop_end\
-                .loc[lambda df: df[pop_age] >= to_counter_age_min + 1]\
-                .loc[lambda df: df[pop_age] <= to_counter_age_max + 1]
-        df_peal_ca[pop_end] -= at_least
         df_other_comp_ca = otbl_other_comp\
             .loc[lambda df: df[pop_age] >= to_counter_age_min]\
             .loc[lambda df: df[pop_age] <= to_counter_age_max]
@@ -1301,14 +1399,17 @@ def calculate_absorbable_in_other_comp(
             # so that the pop.end of respective record can INCREASE.
             # This is true REGARDLESS of whether `peal` is either
             # positive or negative.
-            # 99999999 is the "+Infinity" of this function.
-            to_append = np.repeat([99999999], len(other_comp_values_ca))
+            # 9999999 is the "+Infinity" of this function.
+            to_append = np.repeat([9999999], len(other_comp_values_ca))
 
         df_other_comp_ca[other_comp_J] = to_append
         del df_other_comp_ca[other_comp]
 
         # Absorbables at problematic ages (modification ages)
-        sum_other_comp_ca = sum(to_append)
+        if not other_comp_in_neg:
+            sum_other_comp_ca = 99999999 # prevent overflow
+        else:
+            sum_other_comp_ca = sum(to_append)
         age_is_max = problematic_age.is_max()
         pop_end_query = None
         if age_is_max:
@@ -1334,8 +1435,8 @@ def calculate_absorbable_in_other_comp(
             modifiable_val_in_other_comp =\
                 df_other_comp_m[other_comp].values[0]
             if other_comp_in_neg:
-                # Here lies the maximum value to INCREASE in other_comp
-                # so that the pop.end DECREASES.
+                # Here lies the maximum value one can INCREASE 
+                # in other_comp so that the pop.end DECREASES.
                 modifiable = min(
                     # You cannot increase more than the value that will
                     # make pop.end to be less than `at_least`
@@ -1387,7 +1488,7 @@ def calculate_absorbable_in_other_comp(
             if modifiable == 0:
                 df_other_comp_ca[other_comp_J] = 0
         elif isinstance(to_modify_age, list):
-            if (modifiable == 0).all():
+            if modifiable[0] == 0 and modifiable[1] == 0:
                 df_other_comp_ca[other_comp_J] = 0
         else:
             raise NotImplementedError
@@ -1421,15 +1522,14 @@ def calculate_counter_adjustable_in_comp(
     at_least = get_option('pop.at_least')
     to_counter_age_min = to_counter_age[0]
     to_counter_age_max = to_counter_age[1]
-    df_pop_end_compare = None
-    if comp_in_comp_end:
-        df_pop_end_compare = df_pop_end\
-            .loc[lambda df: df[pop_age] >= to_counter_age_min]\
-            .loc[lambda df: df[pop_age] <= to_counter_age_max]
-    else:
-        df_pop_end_compare = df_pop_end\
-            .loc[lambda df: df[pop_age] >= to_counter_age_min + 1]\
-            .loc[lambda df: df[pop_age] <= to_counter_age_max + 1]
+    to_counter_age_in_pop = calculate_ages_to_counter_in_pop(
+        to_counter_age,
+        comp_in_comp_end
+    )
+
+    df_pop_end_compare = df_pop_end\
+        .loc[lambda df: df[pop_age] >= to_counter_age_in_pop[0]]\
+        .loc[lambda df: df[pop_age] <= to_counter_age_in_pop[1]]
     df_pop_end_compare[pop_end] -= at_least
     df_comp_counter_adjust = df_comp\
         .loc[lambda df: df[pop_age] >= to_counter_age_min]\
@@ -1438,11 +1538,30 @@ def calculate_counter_adjustable_in_comp(
     pop_end_values = df_pop_end_compare[pop_end].values
     comp_values = df_comp_counter_adjust[comp].values
     counter_adjustable = []
+    counter_adjustable_last = []
     to_append = None
+
     if len(comp_values) != len(pop_end_values):
-        # This will never happen since to_counter_age_min is always 
-        # at least -1.
-        raise NotImplementedError
+        # This happends iff:
+        # min(to_counter_age_max + 1, age_max + 1) == age_max + 1
+        # iff:
+        # (not comp_in_comp_end) and use_old_neigh and problematic_age is
+        # big enough to make to_counter_age_max + 1 to go over age_max +1.
+        # e.g. to_counter_age == [97, 100], not comp_in_comp_end,
+        #     so that to_counter_age_in_pop == [98, 100]
+        pop_end_values_last = pop_end_values[-1]
+        pop_end_values = pop_end_values[:-1]
+        comp_values_last = comp_values[-2:]
+        comp_values = comp_values[:-2]
+        if comp_in_neg:
+            ca_0 = pop_end_values_last // 2
+            ca_1 = pop_end_values_last - ca_0
+        else:
+            sum_comp_values_last = sum(comp_values_last)
+            ca_1 = min(pop_end_values_last, comp_values_last[1])
+            ca_0 = min(pop_end_values_last, sum_comp_values_last) - ca_1
+        counter_adjustable_last = [ca_0, ca_1]
+
     zippc = zip(pop_end_values, comp_values)
     for pop_end_minus_at_least_val, comp_val in zippc:
         if pop_end_minus_at_least_val <= 0:
@@ -1488,6 +1607,7 @@ def calculate_counter_adjustable_in_comp(
             # component value will become negative.
             to_append = min(pop_end_minus_at_least_val, comp_val)
         counter_adjustable.append(to_append)
+    counter_adjustable += counter_adjustable_last
 
     if to_absorb_ca is not None:
         aca = pd.DataFrame({
@@ -1659,7 +1779,10 @@ def get_J(
     problematic = I.iloc[0, :] # choose the youngest
     problematic_sex = problematic[pop_sex]
     problematic_age = problematic[pop_age]
-    calculated_pop = poptbl.calculate_pop()
+    calculated_pop_orig = poptbl.calculate_pop()
+    calculated_pop = calculated_pop_orig.copy()
+    age_mask = get_option('age.mask')
+    set_0_at_masked_ages(calculated_pop, pop_age, age_mask, pop_end)
     comp_in_neg = comp in comp_neg # comp is in comp_pos otherwise
     comp_in_comp_end = comp in poptbl.get_comp_end()
 
@@ -1851,8 +1974,15 @@ def get_K(
         dfs_comp = J
     df_comp_modifiable = dfs_comp['records.to_modify']
     df_comp_counter_adjust = dfs_comp['records.to_counter']
+
+    min_mod_age = min(df_comp_modifiable[pop_age].values)
+    min_ca_age = min(df_comp_counter_adjust[pop_age].values)
+    using_old_neigh = min_mod_age < min_ca_age
     df_comp_counter_adjust_reversed = df_comp_counter_adjust\
-        .sort_values(pop_groups, ascending = [True, False])
+        .sort_values(
+            pop_groups, 
+            ascending = [True, True] if using_old_neigh else [True, False]
+        )
     to_modify_val_total = df_comp_modifiable[comp_J].sum()
 
     comp_K = f"{comp}_K"
@@ -1906,6 +2036,7 @@ def get_K(
             else:
                 result.append(0)
     elif method == 'prop':
+        pop_end = poptbl.get_pop_end()
         problematic_sex = df_comp_modifiable[pop_sex].values[0]
         prop_max_age = df_comp_counter_adjust_reversed\
             .iloc[0, :]\
@@ -1916,7 +2047,10 @@ def get_K(
         if not comp_in_comp_end:
             prop_max_age += 1
             prop_min_age += 1
-        calculated_pop = poptbl.calculate_pop()
+        calculated_pop_orig = poptbl.calculate_pop()
+        calculated_pop = calculated_pop_orig.copy()
+        age_mask = get_option('age.mask')
+        set_0_at_masked_ages(calculated_pop, pop_age, age_mask, pop_end)
         pop_end_to_compare_reversed = calculated_pop.copy()\
             .loc[lambda df: df[pop_sex] == problematic_sex]\
             .loc[lambda df: df[pop_age] <= prop_max_age]\
@@ -2023,6 +2157,9 @@ def get_L(
     p_sex = J[pop_sex].values[0]
     p_age = J[pop_age].values
     sum_counter_adjust = K[comp_K].sum()
+    if sum_counter_adjust == 0:
+        J[comp_J] = 0
+        sum_mod_val = J[comp_J].sum()
     assert sum_mod_val == sum_counter_adjust, \
         f"The total modification value, {sum_mod_val}, " +\
         f"at sex {p_sex} age(s) {p_age} and " +\
@@ -2072,6 +2209,33 @@ def get_other_comp(tbl: PopTable, comp: str):
     
     return other_comp
 
+def set_0_at_masked_ages(dat, age_name, age_mask, val_name):
+    '''
+    Given `dat[val_name]`, if the value of `dat[age_name]` in the same 
+    record belongs to `age_mask`, then modify the value at `dat[val_name]`
+    to 0.
+
+    Arguments
+    ---------
+    * `dat`: a `pandas.DataFrame` that contains columns named `age_name`
+        and `val_name`.
+    * `age_name`: a `str`; the column name that contains values for the
+        filter 
+    * `age_mask`: a `list` of `str`; the list of all items that defines a
+        filter
+    * `val_name`: a `str`; the column name that is to be mutated based on
+        the filter
+    '''
+
+    if age_mask != []:
+        masked_val = []
+        for ind, row in dat.iterrows():
+            if str(row[age_name]) in age_mask:
+                masked_val.append(0)
+            else:
+                masked_val.append(row[val_name])
+        dat[val_name] = masked_val
+
 def return_goption_if_None(opname: str, argument):
     '''
     Return the global option value of `opname` if `argument` is None,
